LIBRARY IEEE; 
USE IEEE.std_logic_1164.ALL;  
USE IEEE.std_logic_arith.ALL;  
LIBRARY lpm;
USE lpm.lpm_components.all;
USE WORK.types.ALL;

ENTITY MipsMemory IS
 GENERIC (filename: STRING := "sqrt.mif");
    PORT (clk:     IN bit_t;
          reset:   IN bit_t;
          req: 	   IN bit_t; 
  		  rw:  	   IN bit_t;
          hold:    OUT bit_t;
          addr:    IN halfword_t;
          data:    INOUT halfword_t
         ); 
END MipsMemory;

ARCHITECTURE MaxPlusII OF MipsMemory IS
  TYPE StateType IS (W,A,B);
  SIGNAL state,nextstate: StateType;
  SIGNAL Wen: std_logic;

  COMPONENT lpm_ram_io
    GENERIC (LPM_WIDTH: POSITIVE;
      LPM_TYPE: STRING := "LPM_RAM_IO";
      LPM_WIDTHAD: POSITIVE;
      LPM_NUMWORDS: STRING := "UNUSED";
      LPM_FILE: STRING := "UNUSED";
      LPM_INDATA: STRING := "REGISTERED";
      LPM_ADDRESS_CONTROL: STRING := "REGISTERED";
      LPM_OUTDATA: STRING := "REGISTERED";
      LPM_HINT: STRING := "UNUSED");
    PORT (address: IN STD_LOGIC_VECTOR(LPM_WIDTHAD-1 DOWNTO 0);
      we: IN STD_LOGIC;
      inclock: IN STD_LOGIC := '1';

      outclock: IN STD_LOGIC := '1';
      outenab: IN STD_LOGIC := '1';
      memenab: IN STD_LOGIC := '1';
      dio: INOUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0));
  END COMPONENT;

  SIGNAL i_data: halfword_t;

BEGIN
  
  nextstate <= W WHEN reset = '0' ELSE
               A WHEN state = W AND req = '1' ELSE
               B WHEN state = A AND req = '1' ELSE
               W;

  hold <= '1' WHEN req = '1' AND state /= B ELSE '0';

  PROCESS(clk) 
  BEGIN
    IF clk'EVENT AND clk = '0' THEN
      state <= nextstate;
    END IF;
  END PROCESS;


  Wen <= NOT rw;

  memory: LPM_RAM_IO
    GENERIC MAP (lpm_width => 16,
				 lpm_widthad => 8,
				 lpm_file => filename,
				 lpm_outdata => "UNREGISTERED" )
	PORT MAP (address => addr(7 DOWNTO 0), we => Wen,
			  inclock => clk, --outclock => clk, 
              outenab => rw, 
			  dio => data);

END MaxPlusII;
