LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;
USE STD.TEXTIO.ALL;
USE WORK.types.ALL;

ENTITY mem2 IS   
  GENERIC (filename : IN STRING  := "";  -- name of memory load file
           n        : IN INTEGER := 0;
           tclock     : IN TIME :=0 ns;
           Tpd : in time := 10 ns);  -- number of hold cycles(wait states)    
  PORT    (Clk  : IN    bit_t;
           Addr : IN    halfword_t := halfword_z;
           Data : INOUT halfword_t := halfword_z;
           RW   : IN    bit_t := 'Z';
           Req  : IN    bit_t := 'Z';
           Hold : OUT   bit_t := '0');
           
END mem2;   

ARCHITECTURE behaviour OF mem2 IS
BEGIN    
  RAM : PROCESS     
    VARIABLE memory  : MemType;  -- memory array  
    VARIABLE index   : NATURAL;  -- index to memory array     
    VARIABLE Int_Hold : bit_t := '0'; 
    VARIABLE HoldCount : INTEGER := 0;
    VARIABLE GoodIndex : BOOLEAN := FALSE; 
  
    -- This procedure loads memory from a file.
    PROCEDURE LoadMemoryFromFile(VARIABLE memory   : OUT MemType;    
                                 CONSTANT filename : IN  STRING) IS
--      FILE     memfile : TEXT IS IN filename;
    file memfile:     text open READ_MODE is filename;
      VARIABLE memline : LINE; 
      VARIABLE tmp     : BIT_VECTOR(15 DOWNTO 0);  
    BEGIN
      FOR i IN memory'RANGE LOOP      
        EXIT WHEN ENDFILE(memfile);        
        READLINE(memfile, memline);       
        READ(memline, tmp);     
        memory(i) := TO_STDLOGICVECTOR(tmp);   
      END LOOP;
    END LoadMemoryFromFile;  

  BEGIN 
    IF (filename /= "") THEN      
      LoadMemoryFromFile(memory, filename);   
    END IF;

    LOOP  -- forever for memory access   
   
      -- Read address on negative clockedge.
      -- If address is undefined (tristated or just X's) then the output will be
      -- undefined. This also happens if the address is changed during a load
      -- or store operation (i.e. while hold is '1').
      IF Clk'EVENT AND Clk = '0' AND Int_Hold = '0' AND NOT IS_X(Addr) THEN
        index := CONV_INTEGER(UNSIGNED(Addr));   
        GoodIndex := TRUE;
      ELSIF (Addr'EVENT AND Int_Hold = '1') OR IS_X(Addr) THEN 
        GoodIndex := FALSE;
      END IF;
      
      -- On positive clock-edge we write data to the memory. 
      IF Clk'EVENT AND Clk = '1' THEN
        -- Make sure the memory isn't driving the bus.
        Data <= halfword_z AFTER Tpd;   
        IF (Req = '1') AND (RW = '0') AND (Int_Hold = '0') THEN  
          IF GoodIndex THEN 
            memory(index) := Data;   
          END IF;
        END IF; 
        
      ELSIF Clk'EVENT AND Clk = '0' THEN
        -- This generates the hold signal. 
        IF (Req = '1') AND (Int_Hold = '0') AND (n /= 0) THEN  
          Int_Hold :='1';
          HoldCount := n;
          Hold <= Int_Hold AFTER Tpd;
        END IF;  
        -- This upholds the hold signal for n clockperiods.
        IF HoldCount > 0 THEN 
          HoldCount := HoldCount - 1;
        ELSE 
          Int_Hold := '0'; 
          Hold <= Int_Hold AFTER Tpd;
        END IF;
        -- For read transactions.
        IF (Req = '1') AND (RW = '1') AND (Int_Hold = '0') THEN       
          IF GoodIndex THEN 
            Data <= memory(index) AFTER Tpd; 
          ELSE 
            Data <= (others => 'U');
          END IF;    
        ELSE         
          Data <= halfword_z AFTER Tpd;   
        END IF;     
      END IF;
      WAIT ON Addr,Req, RW, Clk; 
    END LOOP; 
  END PROCESS RAM;
END behaviour; 
