LIBRARY IEEE; 
USE IEEE.std_logic_1164.ALL;  
USE IEEE.std_logic_arith.ALL;  
USE WORK.types.ALL;

ENTITY MipsSystem IS
  PORT (clk:     IN bit_t;
        reset:   IN bit_t;
        req:     INOUT bit_t;
        rw:      INOUT bit_t;
        hold:    INOUT bit_t;
        addr:    INOUT halfword_t;
        data:    INOUT halfword_t;
        memdata: INOUT halfword_t;
        memclk : INOUT bit_t;

		-- These are for the displays.
        digit1: OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
        digit2: OUT STD_LOGIC_VECTOR(7 DOWNTO 0)

        -- These are for fun.
        --debug1:  OUT halfword_t;
        --debug2:  OUT halfword_t;
        --debug3:  OUT halfword_t
       );
END ENTITY;

ARCHITECTURE structure OF MipsSystem IS 

  COMPONENT MiniMIPS   --  A simple Mini-MIPS processor.
    PORT (clk:     IN bit_t;
          reset:   IN bit_t;
          req:     OUT bit_t;
          rw:      OUT bit_t;
          hold:    IN bit_t;
          addr:    OUT halfword_t;
          data:    INOUT halfword_t
          --debug1:  OUT halfword_t;
          --debug2:  OUT halfword_t;
          --debug3:  OUT halfword_t
         );
  END COMPONENT;

  COMPONENT MipsMemory 
    GENERIC (filename: STRING := "sqrt.mif");
    PORT (clk:     IN bit_t;
          reset:   IN bit_t;
          req: 	   IN bit_t; 
  	      rw:  	   IN bit_t;
          hold:    OUT bit_t;
          addr:    IN halfword_t;
          data:    INOUT halfword_t
         );
  END COMPONENT;

  SIGNAL clkcount: UNSIGNED(21 DOWNTO 0);
  SIGNAL internclk: bit_t;
  SIGNAL internrw, internreq, internhold: bit_t;
  SIGNAL ONE: bit_t;
  SIGNAL internaddr: halfword_t;
  SIGNAL display1,display2: std_logic_vector(3 downto 0);
 
  --signal debug1, debug2, debug3 : halfword_t;

BEGIN
  memclk <= internclk;

  -- divide clock down from 25 Mhz
  internclk <= clkcount(1);

  cpu: MiniMIPS
    PORT MAP (
      clk => internclk,
      reset => reset,
      req => internreq,
      rw => internrw,
      hold => hold,
      addr => internaddr,
      data => data
    );
  -- These make the compiler use tri-state-buffers on the pins.
  ONE <= '1';
  rw <= internrw WHEN ONE = '1' ELSE 'Z';
  req <= internreq WHEN ONE = '1' ELSE 'Z';
  hold <= internhold WHEN ONE = '1' ELSE 'Z';
  addr(00) <= internaddr(00) when internreq='1' else 'Z';
  addr(01) <= internaddr(01) when internreq='1' else 'Z';
  addr(02) <= internaddr(02) when internreq='1' else 'Z';
  addr(03) <= internaddr(03) when internreq='1' else 'Z';
  addr(04) <= internaddr(04) when internreq='1' else 'Z';
  addr(05) <= internaddr(05) when internreq='1' else 'Z';
  addr(06) <= internaddr(06) when internreq='1' else 'Z';
  addr(07) <= internaddr(07) when internreq='1' else 'Z';
  addr(08) <= internaddr(08) when internreq='1' else 'Z';
  addr(09) <= internaddr(09) when internreq='1' else 'Z';
  addr(10) <= internaddr(10) when internreq='1' else 'Z';
  addr(11) <= internaddr(11) when internreq='1' else 'Z';
  addr(12) <= internaddr(12) when internreq='1' else 'Z';
  addr(13) <= internaddr(13) when internreq='1' else 'Z';
  addr(14) <= internaddr(14) when internreq='1' else 'Z';
  addr(15) <= internaddr(15) when internreq='1' else 'Z';

  mem: MipsMemory
      GENERIC MAP ("fib.mif")
      --GENERIC MAP ("sqrt.mif")
      --GENERIC MAP ("pipetest.mif")
    PORT MAP (memclk,reset,req,rw,internhold,addr,memdata);

  PROCESS(clk)
  BEGIN
    IF clk'EVENT AND clk = '1' THEN
      clkcount <= clkcount + 1;
    END IF;
  END PROCESS;

  -- The next bit display the least significant byte of the address on the display
  -- (FLEX_DIGIT 1 and 2).

  display1 <= internaddr(7 DOWNTO 4);
--  display1 <= CONV_STD_LOGIC_VECTOR(clkcount(25 DOWNTO 22),4);
  display2 <= internaddr(3 DOWNTO 0);
  
  PROCESS(display1)
  BEGIN
    digit1(0) <= '1'; digit1(1) <= '1'; digit1(2) <= '1'; digit1(3) <= '1'; digit1(4) <= '1'; digit1(5) <= '1'; digit1(6) <= '1'; digit1(7) <= '1'; 
	CASE display1 IS
    WHEN "0000" => digit1(0) <= '0'; digit1(1) <= '0'; digit1(2) <= '0'; digit1(3) <= '0'; digit1(4) <= '0'; digit1(5) <= '0';
	WHEN "0001" => digit1(1) <= '0'; digit1(2) <= '0';
    WHEN "0010" => digit1(0) <= '0'; digit1(1) <= '0'; digit1(3) <= '0'; digit1(4) <= '0'; digit1(6) <= '0';
	WHEN "0011" => digit1(0) <= '0'; digit1(1) <= '0'; digit1(2) <= '0'; digit1(3) <= '0'; digit1(6) <= '0';
	WHEN "0100" => digit1(1) <= '0'; digit1(2) <= '0'; digit1(5) <= '0';digit1(6) <= '0';
   	WHEN "0101" => digit1(0) <= '0'; digit1(2) <= '0'; digit1(3) <= '0'; digit1(5) <= '0'; digit1(6) <= '0';
	WHEN "0110" => digit1(0) <= '0'; digit1(2) <= '0'; digit1(3) <= '0'; digit1(4) <= '0'; digit1(5) <= '0'; digit1(6) <= '0';
	WHEN "0111" => digit1(0) <= '0'; digit1(1) <= '0'; digit1(2) <= '0'; 
	WHEN "1000" => digit1(0) <= '0'; digit1(1) <= '0'; digit1(2) <= '0'; digit1(3) <= '0'; digit1(4) <= '0'; digit1(5) <= '0'; digit1(6) <= '0';
	WHEN "1001" => digit1(0) <= '0'; digit1(1) <= '0'; digit1(2) <= '0'; digit1(5) <= '0'; digit1(6) <= '0';
	WHEN "1010" => digit1(0) <= '0'; digit1(1) <= '0'; digit1(2) <= '0'; digit1(4) <= '0'; digit1(5) <= '0'; digit1(6) <= '0';
	WHEN "1011" => digit1(2) <= '0'; digit1(3) <= '0'; digit1(4) <= '0'; digit1(5) <= '0'; digit1(6) <= '0';
	WHEN "1100" => digit1(0) <= '0'; digit1(3) <= '0'; digit1(4) <= '0'; digit1(5) <= '0'; 
	WHEN "1101" => digit1(1) <= '0'; digit1(2) <= '0'; digit1(3) <= '0'; digit1(4) <= '0'; digit1(6) <= '0';
	WHEN "1110" => digit1(0) <= '0'; digit1(3) <= '0'; digit1(4) <= '0'; digit1(5) <= '0'; digit1(6) <= '0';
	WHEN "1111" => digit1(0) <= '0'; digit1(4) <= '0'; digit1(5) <= '0'; digit1(6) <= '0';
    WHEN OTHERS =>
    END CASE;
  END PROCESS;

  PROCESS(display2)
  BEGIN
    digit2(0) <= '1'; digit2(1) <= '1'; digit2(2) <= '1'; digit2(3) <= '1'; digit2(4) <= '1'; digit2(5) <= '1'; digit2(6) <= '1'; digit2(7) <= '1'; 
	CASE display2 IS
    WHEN "0000" => digit2(0) <= '0'; digit2(1) <= '0'; digit2(2) <= '0'; digit2(3) <= '0'; digit2(4) <= '0'; digit2(5) <= '0';
	WHEN "0001" => digit2(1) <= '0'; digit2(2) <= '0';
    WHEN "0010" => digit2(0) <= '0'; digit2(1) <= '0'; digit2(3) <= '0'; digit2(4) <= '0'; digit2(6) <= '0';
	WHEN "0011" => digit2(0) <= '0'; digit2(1) <= '0'; digit2(2) <= '0'; digit2(3) <= '0'; digit2(6) <= '0';
	WHEN "0100" => digit2(1) <= '0'; digit2(2) <= '0'; digit2(5) <= '0';digit2(6) <= '0';
   	WHEN "0101" => digit2(0) <= '0'; digit2(2) <= '0'; digit2(3) <= '0'; digit2(5) <= '0'; digit2(6) <= '0';
	WHEN "0110" => digit2(0) <= '0'; digit2(2) <= '0'; digit2(3) <= '0'; digit2(4) <= '0'; digit2(5) <= '0'; digit2(6) <= '0';
	WHEN "0111" => digit2(0) <= '0'; digit2(1) <= '0'; digit2(2) <= '0'; 
	WHEN "1000" => digit2(0) <= '0'; digit2(1) <= '0'; digit2(2) <= '0'; digit2(3) <= '0'; digit2(4) <= '0'; digit2(5) <= '0'; digit2(6) <= '0';
	WHEN "1001" => digit2(0) <= '0'; digit2(1) <= '0'; digit2(2) <= '0'; digit2(5) <= '0'; digit2(6) <= '0';
	WHEN "1010" => digit2(0) <= '0'; digit2(1) <= '0'; digit2(2) <= '0'; digit2(4) <= '0'; digit2(5) <= '0'; digit2(6) <= '0';
	WHEN "1011" => digit2(2) <= '0'; digit2(3) <= '0'; digit2(4) <= '0'; digit2(5) <= '0'; digit2(6) <= '0';
	WHEN "1100" => digit2(0) <= '0'; digit2(3) <= '0'; digit2(4) <= '0'; digit2(5) <= '0'; 
	WHEN "1101" => digit2(1) <= '0'; digit2(2) <= '0'; digit2(3) <= '0'; digit2(4) <= '0'; digit2(6) <= '0';
	WHEN "1110" => digit2(0) <= '0'; digit2(3) <= '0'; digit2(4) <= '0'; digit2(5) <= '0'; digit2(6) <= '0';
	WHEN "1111" => digit2(0) <= '0'; digit2(4) <= '0'; digit2(5) <= '0'; digit2(6) <= '0';
    WHEN OTHERS =>
    END CASE;
  END PROCESS;


END structure;
