-------------------------------------------------------------------------------
-- singlebitALU.vhd
-- ----------------
-- This is a single bit-slice ALU using a tree-like structure of depth 3 using
-- 6 mux'es for the different functions. Some signals must be controlled 
-- outside this single-bit ALU since it is optimized for speed - not size.
--
-- Author      :  c973811 Mikael Lund Jepsen
--             :  c973816 Emil Heinze
--             :  c973431 Rune M. Barnkob
--
-- Last Update :  2001/02/01 - Fast tree-like structure used, more control req.
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use WORK.types.all;

entity singlebitALU is port (
  Binv: in  bit_t;
  Cin:  in  bit_t;
  op:   in  std_logic_vector(5 downto 0);
  A:    in  bit_t;
  B:    in  bit_t;
  R:    out bit_t;
  prevbit: in bit_t;
  nextbit: in bit_t;
  slt:  in  bit_t;
  Radd: out bit_t; -- bypass for the adder result
  Cout: out bit_t
  );
end entity singlebitALU;

architecture behaviour of singlebitALU is
  signal mux1: bit_t;
  signal mux2: bit_t;
  signal mux3: bit_t;
  signal mux4: bit_t;
  signal mux5: bit_t;
  signal mux6: bit_t;
  
  signal r_add: bit_t;
  signal r_log: bit_t;
begin
  mux1 <= prevbit when op(0)='1' else nextbit;
  mux2 <= slt when op(1)='1' else (A and B);
  mux3 <= (A or B) when op(2)='1' else (A xor B);
  mux4 <= mux1 when op(3)='1' else mux2;
  mux5 <= mux3 when op(4)='1' else r_add;
  mux6 <= mux4 when op(5)='1' else mux5;
  R <= mux6;

  r_add <= A xor (B xor Binv) xor Cin;
  Radd <= r_add;
  
  Cout <= (A and (B xor Binv)) or (A and Cin) or ((B xor Binv) and Cin);
end;
