#ifndef DISASM_H
#define DISASM_H
///////////////////////////////////////////////////////////////////////////////
// disasm.h
// --------
//
// Dis-assembler for the MiniMIPS CPU made in 49280 Computer Organization and
// design. This DisAssembler will translate a single 16-bit instruction into a
// string which is written to the stream supplied. 
//
// There are at least two ways to use the DisAssembler - either as a kind of
// casting a WORD to an instruction in a streamed output like this:
// 	"cout << DisAsm(instr, address) << endl;"
// or as an object dedicated to being a translator, like this:
// 	DisAsm da;
// 	for (lp=0; lp<MEM_WORDSIZE; lp++) cout << da.disasm(mem[lp], lp, DisAsm::has_base) << endl;
// 
// Flags may be combined as to form a specific output for your purposes:
// 	disasm(_,_,DisAsm::has_addr|DisAsm::opcode_bin|DisAsm::print_addr) - to print addr and opcode in binary
// 
// If no address is supplied, the branch-targets are relative values, otherwise they are real targets.
//
// History:
//   2000-10-16	1.0	Rune M. Barnkob (c973431)
//   2000-11-03	1.4	Added constructor with selectable flags

#include <string>
#include <iostream>
#include "minimips.h"

class DisAsm {
	private:
		WORD intopcode;
		WORD intat;
		int intflags;
	public:
		// construct a new disassembler 
		DisAsm();
		DisAsm(WORD i);
		DisAsm(WORD i, WORD a);
		DisAsm(WORD i, WORD a, int flags);
		
		// disassemble "i" at the location "a" using the flags in a (from the enum).
		string disasm(WORD i, WORD a=0, int flags=0);
		friend ostream& operator<<(ostream&, DisAsm);

		enum { opcode_bin=1, opcode_hex=2, has_addr=4, print_addr=8 };
};
#endif