#ifndef MINIMIPS_H
#define MINIMIPS_H
///////////////////////////////////////////////////////////////////////////////
// minimips.h
// ----------
//
// This is the primary header file for the mini-MIPS project for the course
// 49280 Computer Organization and Design. The header defines the basic 
// bit-field based records for the mini-MIPS CPU instruction set.
//
// Remember that the MiniMIPS only supports WORD's, so all memory cells are
// words.
//
// History
//   2000-10-16	1.0	Created this file
//   2000-10-30	1.1	Fix for SPARC doing bit-field assigns from MSB
//   2000-11-01	1.2	Fixed unsigned immediate to signed immediate (stupido)

// Declare a signed 16-bit integer
typedef signed short int INT;
// Declare a word as a 16-bit unsigned integer
typedef unsigned short int WORD;
// Declare a native character
typedef WORD CHAR;
// Remember the number of registers
#define REG_COUNT 8


// Define this placeholder if your compiler starts from MSB.
// If not declared, the code assumes that the compiler
// starts from LSB (least significant bit) and 16 bit upwards
#define _BITFIELD_START_FROM_MSB

#ifdef BITFIELD_START_FROM_MSB

// R-type instruction
struct oc_reg {
	unsigned op: 3;
	unsigned rs: 3;
	unsigned rt: 3;
	unsigned rd: 3;
	unsigned func: 4;
};

// Immediate instruction
struct oc_imm {
	unsigned op: 3;
	unsigned rs: 3;
	unsigned rt: 3;
	signed imm: 7;
};

// Memory instruction
struct oc_mem {
	unsigned op: 3;
	unsigned rs: 3;
	unsigned rt: 3;
	signed offset: 7;
};

// Branch type instruction	(same as memory)
struct oc_bra {
	unsigned op: 3;
	unsigned rs: 3;
	unsigned rt: 3;
	signed offset: 7;
};

// Jump type instruction	
struct oc_jmp {
	unsigned op: 3;
	unsigned target: 13;
};

#else

// R-type instruction
struct oc_reg {
	unsigned func: 4;
	unsigned rd: 3;
	unsigned rt: 3;
	unsigned rs: 3;
	unsigned op: 3;
};

// Immediate instruction
struct oc_imm {
	signed imm: 7;
	unsigned rt: 3;
	unsigned rs: 3;
	unsigned op: 3;
};

// Memory instruction
struct oc_mem {
	signed offset: 7;
	unsigned rt: 3;
	unsigned rs: 3;
	unsigned op: 3;
};

// Branch type instruction	(same as memory)
struct oc_bra {
	signed offset: 7;
	unsigned rt: 3;
	unsigned rs: 3;
	unsigned op: 3;
};

// Jump type instruction	
struct oc_jmp {
	unsigned target: 13;
	unsigned op: 3;
};

#endif

// Declare a union to access all the different instruction formats
union oc {
	oc_reg reg;
	oc_imm imm;
	oc_mem mem;
	oc_bra bra;
	oc_jmp jmp;
	WORD opcode;
};

#endif