#ifndef SCANNER_H
#define SCANNER_H
///////////////////////////////////////////////////////////////////////////////
// scanner.h
// ---------
//
// This is the header-file for the mini-MIPS assemblers scanner.
//
// History:
//   2000-10-16	1.0	Modified from the R1-assembler by Rune M. Barnkob (c973431)

#include <string>
#include <map>
#include "minimips.h"


// All the tokens, you may encounter during scanning.
enum ScTokens {
	TT_EOL,			// end-of-line
	TT_ID,			// identifier
	TT_CONST,		// number
	TT_STRING,		// string
	// Paranthesis and brackets '[' ']' '(' ')'
	TT_LB, TT_RB, TT_LP, TT_RP,
	// Operators: ',' '.' '=' ':' '@' '+' '-' '*'
	TT_COMMA, TT_DOT, TT_EQ, TT_COLON,
	TT_AT, TT_PLUS, TT_MINUS, TT_MULT,
	// Instructions
	TT_SLL, TT_SRL, TT_SRA, TT_JR, TT_ADD, TT_SUB, TT_AND, TT_OR, TT_XOR, TT_SLT, TT_SLTU,
	TT_ADDI, TT_J, TT_JAL, TT_BEQ, TT_BNE, TT_LW, TT_SW,
	// registers - $<num> - register number can be read from getWord().
	TT_REG,
	// storage-specifiers
	TT_WORD, TT_ASCII, TT_ASCIIZ,
	// directives
	TT_SEG, TT_END, TT_ASSUME, TT_CODE, TT_DATA, TT_ORG
};


class Scanner {
	public:
		Scanner(bool = true);
		~Scanner();

		// Initialise scanner for new line.
		// All internal fields are reset
		void newLine( string );

		// Return the current lexeme
		string getLex();

		// Get a value from the input (if current is val)
		// May throw an exception if incorrect token
		WORD getWord();

		// Read the next token from the line
		// The type of the token is returned
		ScTokens next();

		// Returns the current token type.
		ScTokens curTok() { return tok; }

 protected:
		char line[256];
		int linepos;
		WORD val;
		string lex;
		ScTokens tok;
		bool chk_kw;

		void scId();
		void scRegister();
		void scVal();
		void scString();
		void check_keyword();

};

#endif
