-------------------------------------------------------------------------------
-- singlebitALU.vhd
-- ----------------
-- This is a single bit ALU almost as described in Patterson & Hennessy, p. 238.
--
-- Author      :  c973811 Mikael Lund Jepsen
--             :  c973816 Emil Heinze
--             :  c973431 Rune M. Barnkob
--
-- Last Update :  2001/01/29
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use WORK.types.all;

entity singlebitALU is port (
  Binv: in  bit_t;
  Cin:  in  bit_t;
  l_op:   in  std_logic_vector(1 downto 0);
  a_op:   in  std_logic_vector(1 downto 0);
  A:    in  bit_t;
  B:    in  bit_t;
  R:    out bit_t;
  prevbit: in bit_t;
  nextbit: in bit_t;
  slt:  in  bit_t;
  Radd: out bit_t; -- bypass for the adder result
  Cout: out bit_t
  );
end entity singlebitALU;

architecture behaviour of singlebitALU is
  signal r_add: bit_t;
  signal r_log: bit_t;
begin
  r_log <= A and B when l_op="00" else
           A or B  when l_op="01" else
           A xor B when l_op="10" else
           slt;

  R <= r_log   when a_op="00" else
       nextbit when a_op="01" else
       prevbit when a_op="10" else
       r_add;
  
  r_add <= A xor (B xor Binv) xor Cin;
  Cout <= (A and (B xor Binv)) or (A and Cin) or ((B xor Binv) and Cin);
  Radd <= r_add;
  
end;
