-- -----------------------------------------------------------------------------
--
--  Title      :  Testbench for task two of the Mini-MIPS project.
--             :
--  Developers :  Niels Haandbk -- c958307@student.dtu.dk
--             :
--  Purpose    :  This design contains a testbench for the CPU build in task 
--             :  two. It uses two external memory banks, one for instructions
--             :  and one for data.                           .                
--             :
--  Revision   :  1.0    09-9-99     Initial version
--              
-- -----------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE WORK.types.ALL;

ENTITY mips IS
END mips;

ARCHITECTURE structure OF mips IS
    COMPONENT clock 
        GENERIC (period:        TIME := 50 ns);
        PORT (clk:              OUT std_logic := '0');
    END COMPONENT;

    COMPONENT mem2 
        GENERIC (filename:      STRING  := "";      
                 n:             NATURAL := 0;       
                 Tclock:        TIME := 50 ns);     
        PORT (clk:        IN    bit_t;              
              addr:       IN    halfword_t;         
              data:       INOUT halfword_t;       
              rw:         IN    bit_t;              
              req:        IN    bit_t;              
              hold:       OUT   bit_t);             
    END COMPONENT;   

    COMPONENT cpu2 
        PORT (clk :       IN    bit_t;           
              reset:      IN    bit_t;           
              instaddr:   OUT   halfword_t;      
              inst:       INOUT halfword_t;      
              instreq:    OUT   bit_t;           
              instrw:     OUT   bit_t;           
              insthold:   IN    bit_t;           
              dataaddr:   OUT   halfword_t;      
              data:       INOUT halfword_t;      
              datareq:    OUT   bit_t;           
              datarw:     OUT   bit_t;           
              datahold:   IN    bit_t);          
    END COMPONENT;

    SIGNAL clk:      bit_t;
    SIGNAL reset:    bit_t;

    SIGNAL instaddr: halfword_t;
    SIGNAL inst_bus: halfword_t;
    SIGNAL instreq:  bit_t;
    SIGNAL instrw:   bit_t;
    SIGNAL insthold: bit_t;

    SIGNAL dataaddr: halfword_t;
    SIGNAL data_bus: halfword_t;
    SIGNAL datareq:  bit_t;
    SIGNAL datarw:   bit_t;
    SIGNAL datahold: bit_t;

BEGIN
    reset <= '1', 'L' AFTER 57 ns;

    SysClk: clock
        PORT MAP (clk      => clk);

    Processor: cpu2
        PORT MAP (clk      => clk,
                  reset    => reset,
                  instaddr => instaddr,
                  inst     => inst_bus,
                  instreq  => instreq,
                  instrw   => instrw,
                  insthold => insthold,
                  dataaddr => dataaddr,
                  data     => data_bus,
                  datareq  => datareq,
                  datarw   => datarw,
                  datahold => datahold);

    InstMem: mem2
        PORT MAP (addr     => instaddr,
                  data     => inst_bus,
                  req      => instreq,
                  rw       => instrw,
                  hold     => insthold,
                  clk      => clk);

    DataMem: mem2
        PORT MAP (addr     => dataaddr,
                  data     => data_bus,
                  req      => datareq,
                  rw       => datarw,
                  hold     => datahold,
                  clk      => clk);

END structure;

-------------------------------------------------------------------------------- 
-- A configuration for the testbench. This configuration will make the cpu load
-- the file "fib2.src.bin" when a simulation is started.
-------------------------------------------------------------------------------- 

CONFIGURATION test_fib OF mips IS
    FOR structure
        FOR SysClk: clock
            USE ENTITY WORK.clock(behaviour)
                GENERIC MAP(period => 50 ns);
        END FOR;

        FOR Processor: cpu2
                USE ENTITY WORK.cpu2(structure);
        END FOR;
    
        FOR InstMem: mem2
            USE ENTITY WORK.mem2(behaviour)
                GENERIC MAP(filename => "..\minimips_prog\fib2.src.bin",
                            n        => 0,
                            Tclock   => 50 ns);
        END FOR;
    
        FOR DataMem: mem2
            USE ENTITY WORK.mem2(behaviour)
                GENERIC MAP(filename => "..\minimips_prog\fib2.src.bin",
                            n        => 1,
                            Tclock   => 50 ns);
        END FOR;
    END FOR;
END test_fib;

-------------------------------------------------------------------------------- 
-- A configuration for the testbench. This configuration will make the cpu load
-- the file "sqrt2.src.bin" when a simulation is started.
-------------------------------------------------------------------------------- 

CONFIGURATION test_sqrt OF mips IS
    FOR structure
        FOR SysClk: clock
            USE ENTITY WORK.clock(behaviour)
                GENERIC MAP(period => 50 ns);
        END FOR;

        FOR Processor: cpu2
                USE ENTITY WORK.cpu2(structure);
        END FOR;
    
        FOR InstMem: mem2
            USE ENTITY WORK.mem2(behaviour)
                GENERIC MAP(filename => "..\minimips_prog\sqrt2.src.bin",
                            n        => 0,
                            Tclock   => 50 ns);
        END FOR;
    
        FOR DataMem: mem2
            USE ENTITY WORK.mem2(behaviour)
                GENERIC MAP(filename => "..\minimips_prog\sqrt2.src.bin",
                            n        => 1,
                            Tclock   => 50 ns);
        END FOR;
    END FOR;
END test_sqrt;

-------------------------------------------------------------------------------- 
-- A configuration for the testbench. This configuration will make the cpu load
-- the file "pipetest.bin" when a simulation is started.
-------------------------------------------------------------------------------- 

CONFIGURATION test_pipe OF mips IS
    FOR structure
        FOR SysClk: clock
            USE ENTITY WORK.clock(behaviour)
                GENERIC MAP(period => 50 ns);
        END FOR;

        FOR Processor: cpu2
                USE ENTITY WORK.cpu2(structure);
        END FOR;
    
        FOR InstMem: mem2
            USE ENTITY WORK.mem2(behaviour)
                GENERIC MAP(filename => "..\minimips_prog\pipetest.src.bin",
                            n        => 0,
                            Tclock   => 50 ns);
        END FOR;
    
        FOR DataMem: mem2
            USE ENTITY WORK.mem2(behaviour)
                GENERIC MAP(filename => "..\minimips_prog\pipetest.src.bin",
                            n        => 1,
                            Tclock   => 50 ns);
        END FOR;
    END FOR;
END test_pipe;

-------------------------------------------------------------------------------- 
-- A configuration for the testbench. This configuration will make the cpu load
-- the file "fwdtest.src.bin" when a simulation is started.
-------------------------------------------------------------------------------- 

CONFIGURATION test_forward OF mips IS
    FOR structure
        FOR SysClk: clock
            USE ENTITY WORK.clock(behaviour)
                GENERIC MAP(period => 50 ns);
        END FOR;

        FOR Processor: cpu2
                USE ENTITY WORK.cpu2(structure);
        END FOR;
    
        FOR InstMem: mem2
            USE ENTITY WORK.mem2(behaviour)
                GENERIC MAP(filename => "..\minimips_prog\fwdtest.src.bin",
                            n        => 0,
                            Tclock   => 50 ns);
        END FOR;
    
        FOR DataMem: mem2
            USE ENTITY WORK.mem2(behaviour)
                GENERIC MAP(filename => "..\minimips_prog\fwdtest.src.bin",
                            n        => 1,
                            Tclock   => 50 ns);
        END FOR;
    END FOR;
END test_forward;

-------------------------------------------------------------------------------- 
-- A configuration for the testbench. This configuration will make the cpu load
-- the file "br_jump.bin" when a simulation is started.
-------------------------------------------------------------------------------- 

CONFIGURATION test_brjump OF mips IS
    FOR structure
        FOR SysClk: clock
            USE ENTITY WORK.clock(behaviour)
                GENERIC MAP(period => 50 ns);
        END FOR;

        FOR Processor: cpu2
                USE ENTITY WORK.cpu2(structure);
        END FOR;
    
        FOR InstMem: mem2
            USE ENTITY WORK.mem2(behaviour)
                GENERIC MAP(filename => "..\minimips_prog\br_jump.src.bin",
                            n        => 0,
                            Tclock   => 50 ns);
        END FOR;
    
        FOR DataMem: mem2
            USE ENTITY WORK.mem2(behaviour)
                GENERIC MAP(filename => "..\minimips_prog\br_jump.src.bin",
                            n        => 1,
                            Tclock   => 50 ns);
        END FOR;
    END FOR;
END test_brjump;
