-- -----------------------------------------------------------------------------
--
--  Title      :  Testbench for task one of the Mini-MIPS design project.
--             :
--  Developers :  Niels Haandbk -- c958307@student.dtu.dk
--             :
--  Purpose    :  This design contains an architecture for the testbench used in 
--             :  task one of the Mini-MIPS design project. It instantiates one
--             :  component of the type cpu1 (defined in the file cpu1.vhd) and
--             :  a clock generator. 
--             :
-- Notes       :  This testbench is not suitable for task two because the cpu
--             :  entity will change and memory modules will be added.
--             :
--  Revision   :  1.0    25-8-98     Initial version
--             :  1.1    27-8-99     Added more comments and a nice header.
--             :                     All types were changed to use the new MIPS 
--             :                     types from the file types.vhd.
--              
-- -----------------------------------------------------------------------------

-------------------------------------------------------------------------------- 
-- The testbench.
-------------------------------------------------------------------------------- 

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE WORK.types.ALL;

ENTITY testbench IS                      
END testbench;

-------------------------------------------------------------------------------- 
-- A simple architecture for the testbench. The program name used by the cpu
-- entity can be specified here or in a configuration.
-------------------------------------------------------------------------------- 

ARCHITECTURE structure OF testbench IS
    COMPONENT clock 
        GENERIC (period:    TIME := 50 ns);
        PORT (clk:      OUT std_logic := '0');
    END COMPONENT;

    COMPONENT cpu1   
        GENERIC (programname: STRING := "");    -- Name of memory load file.
        PORT (clk:      IN  bit_t;              -- The clock signal.
        reset:          IN  bit_t);             -- The reset signal.
   END COMPONENT;

    SIGNAL clk, reset: bit_t;
   
BEGIN      
   theclock: clock
        PORT MAP (clk);

   theCPU: cpu1
        PORT MAP (clk, reset);
END structure;

-------------------------------------------------------------------------------- 
-- A configuration for the testbench. This configuration will make the cpu load
-- the file "fib.bin" when a simulation is started.
-------------------------------------------------------------------------------- 

CONFIGURATION testfib OF testbench IS   
    FOR structure
        FOR theclock: clock
            USE ENTITY WORK.clock(behaviour)
            GENERIC MAP (period => 20 ns);
        END FOR;

        FOR theCPU: cpu1
            USE ENTITY WORK.cpu1(specification)
            GENERIC MAP (programname => "fib.bin");
        END FOR;
    END FOR;
END testfib;

-------------------------------------------------------------------------------- 
-- Another configuration for the testbench. This configuration will make the cpu 
-- load the file "sqrt.bin" when a simulation is started.
-------------------------------------------------------------------------------- 

CONFIGURATION testsqrt OF testbench IS   
    FOR structure
        FOR theclock: clock
            USE ENTITY WORK.clock(behaviour)
            GENERIC MAP (period => 20 ns);
        END FOR;

        FOR theCPU: cpu1
            USE ENTITY WORK.cpu1(specification)
            GENERIC MAP (programname => "sqrt.bin");
        END FOR;
    END FOR;
END testsqrt;

-------------------------------------------------------------------------------- 
-- Another configuration for the testbench. This configuration will make the cpu 
-- load the file "testtest.bin" when a simulation is started.
-------------------------------------------------------------------------------- 

CONFIGURATION testtest OF testbench IS   
    FOR structure
        FOR theclock: clock
            USE ENTITY WORK.clock(behaviour)
            GENERIC MAP (period => 20 ns);
        END FOR;

        FOR theCPU: cpu1
            USE ENTITY WORK.cpu1(specification)
            GENERIC MAP (programname => "test.bin");
        END FOR;
    END FOR;
END testtest;

